<?php

function idsSpion() {
    global $ops;

    /* This function is the primary spion (Aanval IDS Engine) processing system */
    
    // Load spion module
	$objModule = new myModule();
	$objModule->loadModule("3");
	
	// Check if spion module is enabled
	if ($objModule->getModuleVar("enabled") == 1) {

		// Create Sensor Object
		$objSensor = new mySensor();
		
		// Retrieve array of sensor id's
		$id = $objSensor->getSensorIDs(3);
	
		// Loop through each activated sensor
	    for ($i=0;$i<count($id);$i++) {
		
	    	// Load sensor information
	        $objSensor->loadSensor($id[$i]);
	
		    // Define Sensor Variables
	        $sensorSID = $objSensor->getSensorVar("sid");
	        $sensorName = $objSensor->getSensorVar("name");
	
	        // Sensor Display
	        procWrite("A", ".. Processing Aanval Sensor");
	        procWrite("A", " . Aanval Sensor ID: $id[$i] (Engine ID: $sensorSID)");
	        procWrite("A", " . Sensor Name: $sensorName");
	    	
	        // Set timers
	        $tmpStart = date("U");
	        $tmpEnd = date("U");
	        $tmpTime = $tmpEnd - $tmpStart;
	
			// Instantiate new processor object
		    $processor = new processor();
	    
		    // Process spion sensors (single)
		    $cnt = $processor->processSpionSensor($id[$i]);
		
	        // Calculate timers
	        $queryEnd = date("U");
	        $queryTime = $queryEnd - $tmpStart;
	        
	    	// End spion processing for this sensor
	        procWrite("A", " . Imported ~$cnt Events (in $queryTime seconds)");       
		}
	}
}

function idsSnort() {
    global $ops;

    /* This function is the primary snort processing system */
    
    // Load snort module
	$objModule = new myModule();
	$objModule->loadModule("1");
	
	// Check if snort module is enabled
	if ($objModule->getModuleVar("enabled") == 1) {

		// Retrieve the max run limit
	    $runNum = ops("site_processorEventRun");
#	    if ($runNum > 250) $runNum = 250;
	
		// Create Sensor Object
		$objSensor = new mySensor();
		
		// Retrieve array of sensor id's
		$id = $objSensor->getSensorIDs(1);
	
		// Set truncateEnabled variable
		$truncateEnabled = ops("site_snortTruncate");
		
		// Loop through each activated sensor
	    for ($i=0;$i<count($id);$i++) {
		
	    	// Load sensor information
	        $objSensor->loadSensor($id[$i]);
	
		    // Define Sensor Variables
	        $sensorSID = $objSensor->getSensorVar("sid");
	        $sensorName = $objSensor->getSensorVar("name");
	
	        // Sensor Display
	        procWrite("A", ".. Processing Snort Sensor");
	        procWrite("A", " . Aanval Sensor ID: $id[$i] (Snort ID: $sensorSID)");
	        procWrite("A", " . Sensor Name: $sensorName");
	    	
	        // Get id of last event insert
	        $eStart = $objSensor->getSensorVar("lastEvent");
	        if (!$eStart) $eStart = 0;
	        
	        // Set timers
	        $tmpStart = date("U");
	        $tmpEnd = date("U");
	        $tmpTime = $tmpEnd - $tmpStart;
	
	        procWrite("A", " . Last Processed Event Stamp: $eStart");    	
	        
	        // Instantiate new snortPort object
			$sp = new sp();
			
            // Perform import of snort to Aanval based on greater than stored timestamp and sensor id
            $data = $sp->processSnortRange($sensorSID, $id[$i], $eStart, $runNum, $truncateEnabled, 0);

            $lastProcessed = $data[0];
            $count = $data[1];
            $cid = $data[2];
            
            if ($cid) {
	            // Perform import of snort to Aanval based on equal to stored timestamp and sensor id (This is a clean up operation to ensure all events for this timestamp have been converted)
	            $data = $sp->processSnortRange($sensorSID, $id[$i], $lastProcessed, $runNum, $truncateEnabled, 1, $cid);			
	
	            $lastProcessed = $data[0];
	            $count = $data[1] + $count;
            }
            
			// Update sensor lastEvent processed
	    	$objSensor->setSensorVar("lastEvent", $lastProcessed);
	
	        // Calculate timers
	        $queryEnd = date("U");
	        $queryTime = $queryEnd - $tmpStart;
	        
	    	// End snort processing for this sensor
	        procWrite("A", " . Converted ~$count Rows (in $queryTime seconds)");       
		}
	}
}

function idsSyslog() {
    global $ops;

    /* This function is the primary syslog processing system */
    
    // Load syslog module
	$objModule = new myModule();
	$objModule->loadModule("2");
	
	// Check if syslog module is enabled
	if ($objModule->getModuleVar("enabled") == 1) {
	    
    	// Begin processing syslog
		procWrite("A", ".. Syslog Daemon Processing Started");	    

		// Instantiate new processor object
	    $processor = new processor();
	
	    // Retrieve List of syslog sensor files from /syslog/ directory
	    $syslogFiles = $processor->getSyslogFiles();
	    
	    // Check for new syslog sensors (create new sensor if necessary)
	    $processor->newSyslogSensorCheck($syslogFiles);
	    
	    // Process syslog sensors (single or multiple)
	    $processor->processSyslogSensors();
	    
    	// Finished processing syslog
		procWrite("A", " . Syslog Processing Finished");	    
	}
}

function idsSyslog2() {
	global $idsDB;

	// Instantiate new processor object
    $processor = new processor();
	
	// Set variables
    $start = date("U");
	
	// Set array of sources
	$queryA = $idsDB->query("select id, path from idsSensor where enabled = 1");
	while (list($q_id, $q_path) = mysql_fetch_row($queryA)) {
		if (strstr($q_path, "/")) $enabledSourceArray[] = $q_id;
	}
	
	if (count($enabledSourceArray) == 0) return;
	
	procWrite("A", ".. Syslog Custom Source Processor Started");	    
	procWrite("A", " . Preparing to process " . count($enabledSourceArray) . " sources");
	
	// Loop through sources
	foreach ($enabledSourceArray as $sid) {

		// Load sensor data
		$objSensor = new mySensor();
		$objSensor->loadSensor($sid);
		
		procWrite("A", " . Processing: " . $objSensor->getSensorVar("path") . " [$sid]");
		
		// Error checking
		if (!@file_exists($objSensor->getSensorVar("path"))) {
			procWrite("A", "ERROR: Source File: " . $objSensor->getSensorVar("path") . " does not exist / cannot be found");
			continue;
		} else if (!@is_file($objSensor->getSensorVar("path"))) {
			procWrite("A", "ERROR: Source File: " . $objSensor->getSensorVar("path") . " is not a file (a directory maybe?)");
			continue;
		}
		
		// Open file handle
		$handle = @fopen($objSensor->getSensorVar("path"), "r");
		
		// Seek to last position
		fseek($handle, $objSensor->getSensorVar("tracker1"));
		
		procWrite("A", " . " . $objSensor->getSensorVar("path") . " with a filesize of " . humanReadableFilesize(filesize($objSensor->getSensorVar("path"))) . " seeking to: " . $objSensor->getSensorVar("tracker1"));

		// Begin processing
		$line = 0;
		if ($handle) {
			
			// Loop through source line by line
			while (!feof($handle)) {
				$buffer = rtrim(fgets($handle, 4096));
				$tracker1 = ftell($handle);

				// If line not blank
				if ($buffer) {
					$line++;
					$dataArray[] = "$buffer";
					$tracker2 = $buffer;
					
					// Exit if processing more than X lines
					if ($line >= 500) break;
				}
			}
			
			// Close file handle
			fclose($handle);
			
			// Write coreStore
#			if ($dataArray) dataStore($sid, $dataArray);
			if ($dataArray) $processor->processSyslogSourceFile($sid, $objSensor->getSensorVar("sid"), $dataArray);
		}

		// Get trackers in case no data processed
		if (!$tracker1) $tracker1 = $objSensor->getSensorVar("tracker1");
		if (!$tracker2) $tracker2 = $objSensor->getSensorVar("tracker2");

		// Calculate byte diff
		$byteDiff = $tracker1 - $objSensor->getSensorVar("tracker1");
		
		// Set trackers
		$objSensor->setSensorVar("tracker1", $tracker1);
		$objSensor->setSensorVar("tracker2", $tracker2);
		
		procWrite("A", " . " . $objSensor->getSensorVar("name") . " processed " . humanReadableFilesize($byteDiff) . " in $line lines");
		
		// Unset used variables
		unset($dataArray, $buffer, $tracker1, $tracker2, $byteDiff);	
	}

	// Time calculations
    $diff = date("U") - $start;

    procWrite("A", " . Source Processor Finished in $diff seconds");
    
	return $out;
}

function archiveTableProcessor() {
    global $ops, $idsDB;

    $epoch = date("U");

	procWrite("B", ".. Archive Processor Started");

	// This is an attempt to resolve *STUCK* archives
	$idsQueryA = $idsDB->query("select id, tablename from " . ops("site_table.idsArchive") . " where enabled = 1 and status = 1 and process < $epoch order by process limit 1");
	while (list($s_id, $s_table) = mysql_fetch_row($idsQueryA)) {
		$id = $s_id;
		$table = $s_table;

		// Get count of table in status 1 mode (processing)
		$idsQueryA = $idsDB->query("select count(id) from $table");
		while (list($s_cnt) = mysql_fetch_row($idsQueryA)) {
			$cnt = $s_cnt;
		}
		
		// If status is processing and count is still 0 - then there is a problem with the archive and it needs to be reset to 0
		if ($cnt == 0) {
			procWrite("B", " . Problem Detected: $table ($id)");
			procWrite("B", " . Attempting to Resolve");
			$idsQueryA = $idsDB->query("drop table $table");
			$idsQueryA = $idsDB->query("update " . ops("site_table.idsArchive") . " set status = 0 where id = $id");
		}
	}
	
	// Retrieve queued archive requests
	$idsQueryA = $idsDB->query("select id, start, end, tablename, type from " . ops("site_table.idsArchive") . " where enabled = 1 and status = 0 and process < $epoch order by process limit 1");
	while (list($s_id, $s_start, $s_end, $s_tablename, $s_type) = mysql_fetch_row($idsQueryA)) {
		$id = $s_id;
		$start = $s_start;
		$end = $s_end;
		$tablename = $s_tablename;
		$type = $s_type;
	}
   
	// Create table for queued archive requests
	if ($tablename) {
		procWrite("B", " . Archive: $tablename");
		
		// Update status of queued archive request
		$idsQueryA = $idsDB->query("update " . ops("site_table.idsArchive") . " set status = 1 where id = $id");

 		// Get count of data
 		$idsQueryA = $idsDB->query("select count(id) from idsMain where epoch >= $start and epoch <= $end");
    	while (list($s_cnt) = mysql_fetch_row($idsQueryA)) {
			$totalCount = $s_cnt;
		}
  		
		if ($type == 1) {
 			procWrite("B", " . Archive Copy Started: $tablename");
			procWrite("B", " . Archive Copy Count: $totalCount");

			// Copy data from idsMain to $tablename
	 		$idsQueryA = $idsDB->query("create table $tablename select * from idsMain where epoch >= $start and epoch <= $end");

		} else {
			
			procWrite("B", " . Archive Move Started: $tablename");
			procWrite("B", " . Archive Move Count: $totalCount");
			procWrite("B", " . Archive Processing");
			
	 		// Copy data from idsMain to $tablename
	 		$idsQueryA = $idsDB->query("create table $tablename select * from idsMain where epoch >= $start and epoch <= $end");	 		
	 							
			// Retrieve array of id's from idsMain which will be used if we are deleting data
	 		$idsQueryA = $idsDB->query("select id from idsMain where epoch >= $start and epoch <= $end");
	    	while (list($s_loc) = mysql_fetch_row($idsQueryA)) {
				$array[] = $s_loc;
			}
			
			// Delete data since we are "Moving"
			deleteEventsByArray($array);
		} 		
		
     	// Update status of queued archive request
		$idsQueryA = $idsDB->query("update " . ops("site_table.idsArchive") . " set status = 2 where id = $id");
 	}
  
   procWrite("B", " . Archive Processor Finished");
 
}

function writeFlashPoll($cnt) {
	global $root_path;

	procWrite("A", ".. Flash Poll System Started");
	
	// Attempt to read in flash poll status file
	if (@file_exists("$root_path" . "poll/flash_poll.sts")) {
	
		// Flash poll status file exists
		
		procWrite("A", " . Updating flash_poll.sts");
		
		// Read file into array
		$fileData = file("$root_path" . "poll/flash_poll.sts");
		
		// Trim and append new count data
		$fileData = trim($fileData[0]) . ",$cnt";
		
		// Replace array with new array which includes new count data
		$fileData = explode(",", $fileData);
		
		// Slice out the last elements
		$fileData = array_slice($fileData, -75);

		// Put the last 10 elements back in an array
		foreach ($fileData as $data) {
			$out .= "$data,";
		}
		
		// Trim ending comma
		$out = substr($out, 0, -1);
		
		// Open file, truncate and write new comma delimited list
	    if ($fp = @fopen("$root_path" . "poll/flash_poll.sts", "w")) {
	
	    	// Write data
	    	fputs ($fp, "$out");
	    	
	    	// Close file handle
	        fclose($fp);
	    }			
		
	} else {
		
		// Flash poll status file does not exist

		procWrite("A", " . Creating flash_poll.sts");

		// Open new file
	    if ($fp = @fopen("$root_path" . "poll/flash_poll.sts", "w")) {
	
	    	// Write procWrite data
	    	fputs ($fp, "$cnt");
	    	
	    	// Close file handle
	        fclose($fp);
	    }	
			
	}
	
	procWrite("A", " . Flash Poll System Finished");

}

function pruneIdsMain() {
    global $idsDB;

    $start = date("U");
    $total = 0;
    $maxRows = ops("site_idsmain_max_rows");
    $limit = ops("site_idsMainMaxPrune");

    if (!$limit) $limit = 750;
    
	if (licenseMonitor() == 2) { $maxRows = 10000; }
    
    
    $eventTotal = scTotalEvents_aanval(5);
    
    procWrite("B", ".. Pruning idsMain Started");
    procWrite("B", " . Maximum Pruning Per Run: $limit");
    procWrite("B", " . Event Total: $eventTotal");
    procWrite("B", " . Maximum Events: $maxRows");

    // If idsMain count is larger than max rows limit -> Purge
	if ($eventTotal > $maxRows) {

        // Get list of ID's for delete from indexes
        $idsQueryA = $idsDB->query("select id from " . ops("site_table.idsMain") . " where id > 1 limit $limit");
        while (list($d_id) = mysql_fetch_row($idsQueryA)) {
            $idList[] = "$d_id";
        }

        // Delete events from idsMain
        $idsQueryA = $idsDB->query("delete from " . ops("site_table.idsMain") . " where id > 1 limit $limit");
        
		// Delete events from indexes
		deleteEventsByArray($idList);
	}

    $end = date("U");
    $seconds = $end - $start;
	
    procWrite("B", " . Pruned $total rows in $seconds seconds");
    procWrite("B", " . Pruning idsMain Finished");

}

function pruneIdsLogs() {
    global $idsDB;

    $start = date("U");
    $total = 0;
    $maxRows = ops("site_idslogs_max_rows");
    $limit = ops("site_idsLogsMaxPrune");
    
    if (!$limit) $limit = 750;
    
    procWrite("B", ".. Pruning idsLogs Started");
    procWrite("B", " . Maximum Pruning Per Run: $limit");

    // Get count of current idsLogs table
    $idsQueryA = $idsDB->query("select count(*) from idsLogs");
    while (list($d_count) = mysql_fetch_row($idsQueryA)) {
        $count = $d_count;
    }

    procWrite("B", " . Log Entries Total: $count");
    procWrite("B", " . Maximum Entries: $maxRows");
    
    // If idsLogs count is larger than max rows limit -> Purge
	if ($count > $maxRows) {
        // Delete events from idsLogs
        $idsQueryA = $idsDB->query("delete from " . ops("site_table.idsLogs") . " where id > 1 limit $limit");
	}

    $end = date("U");
    $seconds = $end - $start;

    procWrite("B", " . Pruned $total rows in $seconds seconds");
    procWrite("B", " . Pruning idsLogs Finished");

}

function hostnameLookup() {
    global $ops, $idsDB;

    /* Hostname lookups are stored in idsHostname table */
    
    // Check if hostname lookups are enabled
    if (ops("site_hostnameLookups")  == 1) {

    	procWrite("B", ".. Hostname Lookups Started");

    	// Check if a reprocess of hostnames was forced through the console
        if (ops("hostnameReprocess") == 1) {

            procWrite("B", " . Hostname Lookup Reset Triggered");

            // Truncate query clears hostname table
            $idsQueryA = $idsDB->query("truncate table idsHostnames");

            // Load Status
        	$objStatus = new myStatus();
        	$objStatus->loadStatus();
        	
        	// Update status variables (last run id)
            $objStatus->setStatusVar("processor_lastHostname", 0);
    	    
    	    // Reset reprocess trigger to 0
            $ops->setOpsVar("hostnameReprocess", 0);
        }

        // Load Status
    	$objStatus = new myStatus();
    	$objStatus->loadStatus();
    	
    	// Get last run id
    	$lastid = $objStatus->getStatusVar("processor_lastHostname");

    	// Set last run id to 0 if empty
    	if (!$lastid) $lastid = 0;

    	// Set $max per run
    	$max = 30;
    	
        procWrite("B", " . Max Lookups Per Run: $max");
        procWrite("B", " . Hostname Lookups Start ID: $lastid");

        $idsQueryA = $idsDB->query("select id, sip from " . ops("site_table.idsMain") . " where id > $lastid limit $max");
        while (list($d_id, $d_ip) = mysql_fetch_row($idsQueryA)) {
                $list[] = $d_ip;
                $track[] = $d_id;
        }

            for ($i=0;$i<count($list);$i++) {

                $id = $track[$i];

                $epoch = date("U");
                unset($exists);

            	$idsQueryA = $idsDB->query("select ip, timestamp from " . ops("site_table.idsHostnames") . " where ip = $list[$i]");
                while (list($d_ip, $d_intime) = mysql_fetch_row($idsQueryA)) {
                    $ip = $d_ip;
                    $exists = 1;
                }

                if (!$exists) {
                	$hostname = gethostbyaddr(acid_to_ip($list[$i]));
        		    $idsQueryA = $idsDB->query("insert into " . ops("site_table.idsHostnames") . " (ip, hostname, timestamp) values ('$list[$i]', '$hostname', '$epoch')");
			        $cnt++;

                    // AI
                    if (ops("ralph_enabled") == 1) {
                    	
                    	// New IP's go here if AI is enabled
                    	ipAnalysisA($list[$i]);
                    	
                        ralphLog("RALPH", "Hostname Lookup", $flag, "$hostname", $list[$i]);
                        $seArray = array("search", "msnbot", "google", "excite", "crawler", "yahoo", "inktomi", "alta-vista", "altavista", "alexa", "atomz", "alltheweb", "infoseek", "northernlight", "wisenut");
                        for ($p=0;$p<count($seArray);$p++) {
                            if (strstr($hostname, $seArray[$p])) {
                                ralphLog("RALPH", "Host is possibly a search engine bot", $flag, $seArray[$p], $list[$i]);
                            }
                            break;
                        }
			            $emailArray = array("mail", "pop", "smtp", "mime");
                        for ($p=0;$p<count($emailArray);$p++) {
                            if (strstr($hostname, $emailArray[$p])) {
                                ralphLog("RALPH", "Host is possibly a mail server or webmail system", $flag, $emailArray[$p], $list[$i]);
                            }
                            break;
                        }
                        $proxyArray = array("prox", "prxy", "anon", "cloak", "socks", "stealth", "wingate");
                        for ($p=0;$p<count($proxyArray);$p++) {
                            if (strstr($hostname, $proxyArray[$p])) {
                                ralphLog("RALPH", "Host is possibly a proxy server or anonymous surfing gateway", $flag, $proxyArray[$p], $list[$i]);
                            }
                            break;
                        }
                        if (strstr($hostname, ".edu")) {
		                	ralphLog("RALPH", "Top Level Domain: .edu", $flag, ".edu", $list[$i]);
 	                	}
                        if (strstr($hostname, ".gov")) {
                                ralphLog("RALPH", "Top Level Domain: .gov", $flag, ".gov", $list[$i]);
                        }
                        if (strstr($hostname, ".org")) {
                                ralphLog("RALPH", "Top Level Domain: .org", $flag, ".org", $list[$i]);
                        }
                        if (strstr($hostname, ".mil")) {
                                ralphLog("RALPH", "Top Level Domain: .mil", $flag, ".mil", $list[$i]);
                        }
                    }
                }
            }

        if (!$id) $id = $lastid;

    	$objStatus = new myStatus();
    	$objStatus->loadStatus();
    	$objStatus->setStatusVar("processor_lastHostname", $id);

    	procWrite("B", " . Hostname Lookups End ID: $id");

    }
    
	procWrite("B", " . Hostname Lookups Finished");
    
}

function oadProcess() {
    global $ops;

    if (ops("site_oadUpload") == 1) {

		procWrite("B", ".. OAD Processing Started");
   	
        // If Reprocess of OAD was forced via console
        if (ops("oadReprocess") == 1) {

            procWrite("B", " . OAD Reset Triggered");

        	$objStatus = new myStatus();
        	$objStatus->loadStatus();
            $objStatus->setStatusVar("processor_lastOAD", 0);
            $ops->setOpsVar("oadReprocess", 0);
        }

    	$objStatus = new myStatus();
    	$objStatus->loadStatus();
    	$lastid = $objStatus->getStatusVar("processor_lastOAD");
    	if (!$lastid) $lastid = 0;

    	if (statusOadLastUploadDiff() > ops("site_oadInterval")) {
    		// send data to aanval.com
    		transmitOAD($lastid);

    	} else {

    	}
    	
		procWrite("B", " . OAD Processing Finished");
    	
    }
}

function transmitOAD($lastid) {
    global $ops;

    $num = ops("site_oadBuffer");
	$key1 = ops("site_registration");

	procWrite("B", " . OAD Transmit ($lastid + $num)");

	$objStatus = new myStatus();
	$objStatus->loadStatus();            
	
    $idsQueryA = $idsDB->query("select id, sip, dip, ename, epoch from idsMain where id > $lastid limit $num");
	while (list($s_id, $s_sip, $s_dip, $s_name, $s_time) = mysql_fetch_row($idsQueryA)) {

		$payload = "$s_sip~~$s_dip~~$s_name~~$s_time";
	    $lastid = $s_id;
        $go = 1;

        if ($go == 1) {
            procWrite("B", " . OAD: $lastid => Sending");

        	// base64 payload data
        	$payload = base64_encode($payload);

            $out = exConnect("oad.aanval.com", "oad_payload.php?payload=$payload&ver=3&certify=$key1");

        	$objStatus->setStatusVar("processor_lastOAD", $lastid);
        }

    	unset($go);
	}
}

function permissionsCheck() {
	global $ops, $root_path;

	// Continue only if permissions checking is enabled
	if (ops("site_permChecking") == 0) return;
	
    procWrite("B", ".. Permissions Checking Started");

    procWrite("B", " . /" . "conf");
    if (!file_exists("$root_path" . "conf")) @mkdir("$root_path" . "conf");
    if (!@chmod("$root_path" . "conf", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "conf");
    }    
    
    procWrite("B", " . /" . "poll/tmp");
    if (!file_exists("$root_path" . "poll/tmp")) @mkdir("$root_path" . "poll/tmp");
    if (!@chmod("$root_path" . "poll/tmp", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "poll/tmp");
    }
    
    procWrite("B", " . /" . "poll/sensorData");
    if (!file_exists("$root_path" . "poll/sensorData")) @mkdir("$root_path" . "poll/sensorData");
    if (!@chmod("$root_path" . "poll/sensorData", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "poll/sensorData");
    }
    
    procWrite("B", " . /" . "tmp");
    if (!file_exists("$root_path" . "tmp")) @mkdir("$root_path" . "tmp");
    if (!@chmod("$root_path" . "tmp", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "tmp");
    }

    procWrite("B", " . /" . "tmp/sc");
    if (!file_exists("$root_path" . "tmp/sc")) @mkdir("$root_path" . "tmp/sc");
    if (!@chmod("$root_path" . "tmp/sc", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "tmp/sc");
    }

    procWrite("B", " . /" . "version");
    if (!file_exists("$root_path" . "version")) @mkdir("$root_path" . "version");
    if(!@chmod("$root_path" . "version", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "version");
    }

    procWrite("B", " . /" . "bulletin");
    if (!file_exists("$root_path" . "bulletin")) @mkdir("$root_path" . "bulletin");
    if(!@chmod("$root_path" . "bulletin", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "bulletin");
    }

    procWrite("B", " . /" . "syslog");
    if (!file_exists("$root_path" . "syslog")) @mkdir("$root_path" . "syslog");
    if(!@chmod("$root_path" . "syslog", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "syslog");
    }
   
    procWrite("B", " . /" . "installation");
    if (!file_exists("$root_path" . "installation")) @mkdir("$root_path" . "installation");
    if(!@chmod("$root_path" . "installation", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "installation");
    }    
    
    procWrite("B", " . /" . "language");
    if (!file_exists("$root_path" . "language")) @mkdir("$root_path" . "language");
    if(!@chmod("$root_path" . "language", 0777)) {
        procWrite("B", " ! Permissions Error: Could not create or set permissions on /" . "language");
    }    
    
    procWrite("B", " . Permissions Checking Finished");

}

function remoteVersionCheck() {
    global $ops, $idsDB, $root_path;

    /* Checks the Aanval.com version website for new available builds */
    
    // Start version checking at least once upon each install / upgrade
    if (@file_exists("$root_path" . "version/version.one")) {
		$ops->setOpsVar("site_versionChecking", 1);    	
		@unlink("$root_path" . "version/version.one");
    }

    // SUB Check Alpha
    $idsQueryA = $idsDB->query("select license from idsLicense where active = 1 and license LIKE '33%'");
    while (list($d_license) = mysql_fetch_row($idsQueryA)) {
		$lic = $d_license;
    }
    
    // SUB Check Beta
    if (subVal($lic, 2) == 1) {
    	$ran = rand(1,2);
    	if ($ran == 1) $sub = 1;
    }
    
    // Check if version checking is enabled
    if (ops("site_versionChecking") == 1) {

        procWrite("B", ".. Version Checking Started");
    	
    	// Retrieve unix epoch
        $epoch = date("U");
        
        // Read in current version /version/verstion.txt file if available
        if ($tmp = @file("$root_path/version/version.txt")) {

        	// Create various variables from array
        	$major = rtrim($tmp[0]);
        	$minor = rtrim($tmp[1]);
        	$name = rtrim($tmp[2]);
        	$build = rtrim($tmp[3]);
        	
        	// Read in available version /version/ver file if available
	        if ($tmp = @file("$root_path/version/ver")) {
	        	
	        	// Create $time variable from array
	        	$time = rtrim($tmp[4]);
	
		        // Check $time variable to see if 15 minutes has passed and it is time to recheck for new versions
	        	if (($time + 900) < $epoch) {
	
		        	// Retrieve site registration code
		            $key1 = ops("site_registration");
		            
		        	procWrite("B", " . Connecting to: update.aanval.com");
	
	        		// Make outbound web connection to aanval server
		        	$versionData = exConnect("update.aanval.com", "updater/aanval_ver.php?major=$major&minor=$minor&build=$build&certify=$key1&ver=2&sub=$sub");

		            // Disable Version Checking if $versionData is empty
		            if ($versionData == "") {
		            	procWrite("B", " ! Version Data is empty or not correct (might be a problem)");
#		            	procWrite("B", " ! Version Checking automatically disabled");
#		            	$ops->setOpsVar("site_versionChecking", 0);            	
		            }

		            // Subscription UPDATE (Strange Place - But we will piggy back off of version checking)
		            if (strstr($versionData, "SUB:")) {
			        	procWrite("B", " . Subscription Data Received");
		            	$tmp = explode(":", $versionData);
		            	$sub = base64_decode($tmp[1]);
					    $idsQueryA = $idsDB->query("select license from idsLicense where license = '$sub'");
					    while (list($d_license) = mysql_fetch_row($idsQueryA)) {
							$found = 1;
					    }	            	    
	            	    if (!$found) {
				        	procWrite("B", " . Updating Subscription");
							$idsQueryA = $idsDB->query("update idsLicense set active = 0 where active = 1");
							$idsQueryA = $idsDB->query("insert into idsLicense (license, licensekey, active, intime) values ('$sub', '0', 1, '$epoch')");
	            	    }
		            }

		            if (!$sub && $versionData != "") {          	
		            	// Create /version/ver file handle (this is the available version file)
			            if ($fp = @fopen("$root_path/version/ver", "w")) {
			            	
							procWrite("B", " . Writing /version/ver tracking file");
			            	
		            		// Write web retrieved version data to /version/ver
		                    fputs ($fp, "$versionData\n$epoch");
		                    
		                    // Close file handle
		                    fclose($fp);
			            }
		            }
		            
	        	} else {
	        		
		        	// Calculate seconds until we make version check
		        	$until = $time + 900 - $epoch;
		        	
			        procWrite("B", " . Next Version Check in $until seconds");
	        	}
	        } else {

	        	// Create /version/ver file handle (fill with emptyness)
	            if ($fp = @fopen("$root_path/version/ver", "w")) {
	            	
					procWrite("B", " . Writing /version/ver tracking file");
	            	
            		// Write web retrieved version data to /version/ver
                    fputs ($fp, "0\n0\n0\n0\n0\n:");
                    
                    // Close file handle
                    fclose($fp);
	            }	        	
	        }
        } else {
        	
    		// Could not open current version file: /version/version.txt    	
        	procWrite("B", " ! File Error: $root_path/version/version.txt could not be read");
        	procWrite("B", " ! Version Checking automatically disabled");
        	$ops->setOpsVar("site_versionChecking", 0);
        }

    }

    /* Download Performed Here */
    
	// Check if the UpdateNow variable == 1
    if (ops("site_versionUpdateNow") == 1) {

    	// Download the .tar.gz file from Aanval - if returns 0 do not continue
        if (updateDownload("www.aanval.com", "/downloads/", "aanval-latest-stable.tar.gz") == 0) {
        

        } else {
        	
        	// Change to the root aanval directory
            @chdir("$root_path/");
            
            // External execute: gunzip .tar.gz
            @exec("gunzip -c aanval-latest-stable.tar.gz | tar -xvf -");
            
            // External execute: rm -f to remove downloaded file
            @exec("rm -f aanval-latest-stable.tar.gz");

        }

        // Disable UpdateNow
        $ops->setOpsVar("site_versionUpdateNow", 0);
    }

	procWrite("B", " . Version Checking Finished");

}

function updateDownload($host, $path, $file) {
    global $root_path, $ops;

    // Default status = 0
    $status = 0;

    procWrite("B", " . Downloading from: $host");
    procWrite("B", " . Receiving file: $file");

    // Make outbound web connection to Aanval server
    $downloadData = exConnect($host, "$path$file");
    
    // Disable Version Checking if $downloadData is empty
    if ($downloadData == "") {
    	procWrite("B", " ! Download Data is empty or not correct (might be a problem)");
    	procWrite("B", " ! Version Checking automatically disabled");
    	$ops->setOpsVar("site_versionChecking", 0);            	
    } else {
	    
    	// Create file handle
	    if ($handle = @fopen("$root_path/$file","w")) {
	    	
		    procWrite("B", " . Writing File: $root_path/$file");
		    
	    	// Dump data into new file handle
	    	fputs($handle, $downloadData);
	    	
	    	// Close file handle
	    	fclose($handle);
	    } else {

	    	// File could not be opened for writing
    		procWrite("B", " ! File Error: $root_path/$file could not be created");
	    	procWrite("B", " ! Version Checking automatically disabled");
	    	$ops->setOpsVar("site_versionChecking", 0);            	
	    }
	    
	    // Check if newly created file exists
	    if (@file_exists("$root_path/$file")) $status = 1;
    }

    return $status;
}

function remoteBulletinCheck() {
    global $ops, $root_path;

    // Check if bulletin checking is enabled
    if (ops("site_bulletinChecking") == 1) {

    	procWrite("B", ".. Bulletin Checking Started");
    	
		// Retrieve unix epoch		
		$epoch = date("U");
		
		// Read in /bulletin/last as an array
        if ($tmp = @file("$root_path/bulletin/last")) {

        	// The only element in the /bulletin/last array is the timestamp
        	$time = $tmp[0];
       	
        } else {
        	// Set $time variable if first run OR /bulletin/last could not be opened	
        	$time = 0;
        }

        // Check $time variable to see if 15 minutes has passed and it is time to recheck for bulletins
        if (($time + 900) < $epoch) {

	        // Retrieve site registration code
        	$key1 = ops("site_registration");
        	
	        procWrite("B", " . Connecting to: update.aanval.com");

        	// Make outbound web connection to aanval server
            $bulletinData = exConnect("update.aanval.com", "bulletin/aanval_info.php?certify=$key1");

            // Disable Bulletin Checking if $bulletinData is empty
            if (!stristr("$bulletinData", "aanval") || $bulletinData == "") {
            	procWrite("B", " ! Bulletin Data is empty or not correct (might be a problem)");
            	procWrite("B", " ! Bulletin Checking automatically disabled");
            	$ops->setOpsVar("site_bulletinChecking", 0);            	
            }
            
            // Create /bulletin/last file handle
            if ($fp = @fopen("$root_path/bulletin/last", "w")) {

            	// Write unix epoch to /bulletin/last
            	fputs ($fp, "$epoch");
            	
            	// Close file handle
                fclose($fp);
                
            } else {
            	
            	// Writing /buletin/last failed
            	procWrite("B", " ! File Error: $root_path/bulletin/last could not be written");
            	procWrite("B", " ! Bulletin Checking automatically disabled");
            	$ops->setOpsVar("site_bulletinChecking", 0);
            }

            if ($fp = @fopen("$root_path/bulletin/info", "w")) {

            	// Write web retrieved bulletin data to /bulletin/info
            	fputs ($fp, "$bulletinData");
            	
            	// Close file handle
                fclose($fp);
                
    	        procWrite("B", " . Data written successfuly");
                
            } else {
            	
            	// Writing /buletin/info failed
            	procWrite("B", " ! File Error: $root_path/bulletin/info could not be written");
            	procWrite("B", " ! Bulletin Checking automatically disabled");
            	$ops->setOpsVar("site_bulletinChecking", 0);
            }
           
        } else {

        	// Calculate seconds until we make bulletin check
        	$until = $time + 900 - $epoch;
        	
	        procWrite("B", " . Next Bulletin Check in $until seconds");
        	
        }
        
        procWrite("B", " . Bulletin Checking Finished");
        
    }
}

function recursiveModuleLoad($null = 0) {
    global $root_path;

    # Read in all available modules recursively
    if (!$this_dir = @dir("$root_path/modules/")) {
        procWrite("A", "conf.php ERROR: \$root_path not correct or $root_path/modules/ not found! Check your conf.php settings");
    } else {
        while ($dir = $this_dir->read()) {
            if ($dir != "." && $dir != "..") {
                if (substr($dir, -4, 4) == "_MOD") {
                    $this_sub = dir("$root_path/modules/$dir");
                    while ($file = $this_sub->read()) {
                        if ($file != "." && $file != "..") {
                            if (substr($file, 0,4) == "inc_") {
                                include_once "../modules/$dir/$file";
                            }
                        }
                    }
                }
            }
        }
    }
}

function actionProcessor() {
	global $ops;

	procWrite("D", ".. Action Processor Started");
	
	// Instantiate new status object
	$objStatus = new myStatus();
	
	// Load status variables
	$objStatus->loadStatus();
	
	// Retrieve last processed action id and add 1 so we don't duplicate an action
	$trackid = $objStatus->getStatusVar("processor_lastAction");
	
	// Set last processed id to 0 if not found
	if (!$trackid) $trackid = 0;
	
	procWrite("D", " . Max Actions Per Run: 250");
	procWrite("D", " . Action Start ID: $trackid");
	
	// Instantiate new action object
	$objAction = new myAction();
	
	# Magic happens here
	// Run last processed id into action matcher system and return the last processed id
	$id = $objAction->actionMatcher($trackid);
	
	// If actions matcher fails to return an id - we don't want to start at 0
	if (!$id) $id = $trackid;
	
	// Update status to reflect last procesed id
	$objStatus = new myStatus();
	$objStatus->loadStatus();
	$objStatus->setStatusVar("processor_lastAction", $id);
	
	procWrite("D", " . Action End ID: $id");
	procWrite("D", " . Action Processor Finished");

}

function actionEmail($actionid, $trackid, $toEmail) {
    global $ops, $idsDB;

    // Build action object and load master details
    $objAction = new myAction();
	$objAction->actionMasterLoad($actionid);
	
	// Set userid variable for query
    $uid = $objAction->getActionMasterVar("userid");

    // Query user details for defaults
    $idsQueryA = $idsDB->query("select email, fname, lname from idsUsers where id = '$uid'");
    while (list($d_email, $d_fname, $d_lname) = mysql_fetch_row($idsQueryA)) {
        $email = $d_email;
        $fname = $d_fname;
        $lname = $d_lname;
    }

    // Query information for details within email
    $idsQueryA = $idsDB->query("select module, proto, sip, dip, sport, dport, etypeid, etypename, enameid, ename, level, payload, sensorid, timestamp from idsMain where id = $trackid");
    while (list($module, $proto, $sip, $dip, $sport, $dport, $etypeid, $etypename, $enameid, $ename, $level, $payload, $sensorid, $timestamp) = mysql_fetch_row($idsQueryA)) {

        // Set sensor name for sensorid variable
        $objSensor = new mySensor;
        $sensorid = $objSensor->getSensorName($sensorid);
    	
        // Retrieve proto name by number
        if ($proto == 1) $proto = "ICMP";
        if ($proto == 6) $proto = "TCP";
        if ($proto == 17) $proto = "UDP";

        // Check if snort and decode payload
        if ($module == 1) { 
        	$payload = packet_payload($payload, 3, 1);
        }        

    	// Set sender name
        if ($objAction->getActionMasterVar("senderName") == "") {
        	$fromName = "$fname $lname";
        } else {
        	$fromName = $objAction->getActionMasterVar("senderName");
        }        
        
    	// Set sender email
        if ($objAction->getActionMasterVar("senderEmail") == "") {
        	$fromEmail = "$email";
        } else {
        	$fromEmail = $objAction->getActionMasterVar("senderEmail");
        }           
        
        // Set subject
        if ($objAction->getActionMasterVar("senderSubject") == "") {
        	$subject = "! Aanval Alert ($ename)";
        } else {

        	// Set subject and run through text replace
        	$subject = actionTextReplace($objAction->getActionMasterVar("senderSubject"), $actionid, $trackid, $module, $proto, $sip, $dip, $sport, $dport, $etypeid, $etypename, $enameid, $ename, $level, $payload, $sensorid, $timestamp);
        }        
        
        // Set body
        if ($objAction->getActionMasterVar("senderBody") == "") {
        
	    	$body = "The following alert was generated by the Aanval Intrusion Detection Console.\n---\n\n";
	    	$body .= "Risk Level: RISK_LEVEL\n\n";
		    $body .= "Source IP: SOURCE_IP : SOURCE_PORT\n";
		    $body .= "Destination IP: DESTINATION_IP : DESTINATION_PORT\n";
		    $body .= "Sensor: SENSOR\n\n";
		    $body .= "Detected Event: SIG_NAME (SIG_NAME_ID)\n\n";
		    $body .= "Detected Event Category: SIG_CLASS (SIG_CLASS_ID)\n\n";
		    $body .= "Aanval ID: AANVALID\n";
		    $body .= "Action ID: ACTIONID\n";
			$body .= "---\n";
		    $body .= "Payload:\n";
		    $body .= "PAYLOAD\n";
	    
        	$body = actionTextReplace($body, $actionid, $trackid, $module, $proto, $sip, $dip, $sport, $dport, $etypeid, $etypename, $enameid, $ename, $level, $payload, $sensorid, $timestamp);
        } else {
        	
        	// Set body and run through text replace
        	$body = actionTextReplace($objAction->getActionMasterVar("senderBody"), $actionid, $trackid, $module, $proto, $sip, $dip, $sport, $dport, $etypeid, $etypename, $enameid, $ename, $level, $payload, $sensorid, $timestamp);
        }
	}

	// Send email
    sendmail($fromEmail, $fromName, $toEmail, $toEmail, $subject, $body);
}

function actionTextReplace($data, $action, $id, $module, $proto, $sip, $dip, $sport, $dport, $etypeid, $etypename, $enameid, $ename, $level, $payload, $sensorid, $timestamp) {

	/* Replaces text for emails which use keywords for replacing text */
	
	// Perform string replace on all keywords
	$data = str_replace("ACTIONID", $action, $data);
	$data = str_replace("AANVALID", $id, $data);
	$data = str_replace("PROTOCOL", $proto, $data);
	$data = str_replace("MODULE", $module, $data);
	$data = str_replace("SOURCE_IP", acid_to_ip($sip), $data);
	$data = str_replace("DESTINATION_IP", acid_to_ip($dip), $data);
	$data = str_replace("SOURCE_PORT", $sport, $data);
	$data = str_replace("DESTINATION_PORT", $dport, $data);
	$data = str_replace("SIG_NAME_ID", $enameid, $data);
	$data = str_replace("SIG_CLASS_ID", $etypeid, $data);
	$data = str_replace("SIG_NAME", $ename, $data);
	$data = str_replace("SIG_CLASS", $etypename, $data);
	$data = str_replace("RISK_LEVEL", $level, $data);
	$data = str_replace("PAYLOAD", $payload, $data);
	$data = str_replace("SENSOR", $sensorid, $data);
	
	// Return data
	return $data;
}

function execCommand($cmd, $trackid) {
    global $ops, $idsDB;

    $idsQueryA = $idsDB->query("select module, proto, sip, dip, sport, dport, etypeid, etypename, enameid, ename, level, payload, sensorid, timestamp from idsMain where id = $trackid");
	while (list($module, $proto, $sip, $dip, $sport, $dport, $etypeid, $etypename, $enameid, $ename, $level, $payload, $sensorid, $timestamp) = mysql_fetch_row($idsQueryA)) {
    	$cmd = actionTextReplace($cmd, $actionid, $trackid, $module, $proto, $sip, $dip, $sport, $dport, $etypeid, $etypename, $enameid, $ename, $level, $payload, $sensorid, $timestamp);
    }
    
    dblog("ACTION_MOD", $cmd, $flag, "Command");

    $fp = popen($cmd, 'r');
    while(!feof($fp)) {
        $read .= fread($fp,1024);
    }
    pclose($fp);

}

function ipAnalysisA($ip) {
	global $ops;

	if (ops("ralph_enabled") == 1) {
		
	    ralphLog("RALPH", "IP Analysis -> Start", $flag, "No Comment", acid_to_ip($ip));
	
	    if (ops("ralph_autoping") == 1) {
	    	ralphLog("RALPH", "IP Analysis -> " . acid_to_ip($ip), $flag, "Auto Ping New Host Issued", acid_to_ip($ip));
	    	addScan($ip, 1);
	    }
	    
	    if (ops("ralph_autoscan") == 1) {
	    	ralphLog("RALPH", "IP Analysis -> " . acid_to_ip($ip), $flag, "Auto Scan New Host Issued", acid_to_ip($ip));
	    	addScan($ip, 2);
	    }
	    
	    if (ops("ralph_autoemail") == 1) {
	    	ralphLog("RALPH", "IP Analysis -> " . acid_to_ip($ip), $flag, "Auto Email New Host Issued", acid_to_ip($ip));
	    	$subject = "AI: New Host Detected";
	    	$message = "IP: " . acid_to_ip($ip) . "\n\n";
			sendmail(ops("ralph_autoemailAddress"), "Aanval Console AI", ops("ralph_autoemailAddress"), ops("ralph_autoemailAddress"), $subject, $message);
	    }
	 
	    ralphLog("RALPH", "IP Analysis -> End", $flag, "No Comment", acid_to_ip($ip));
	}	
}

function ipAnalysisB($ip) {
	global $ops, $idsDB;

	if (ops("ralph_enabled") == 1) {
	
		if (ops("ralph_analysisEnabled") == 1) {
		
		    ralphLog("RALPH", "IP Analysis -> Start", $flag, "No Comment", acid_to_ip($ip));
		
			$idsQueryB = $idsDB->query("select count(loc) from idsReverseIndexSip where data = '$ip'");
			while (list($d_total) = mysql_fetch_row($idsQueryB)) {
				$total = $d_total;
			}
		   
			$ta = ops("ralph_thresholdA");
			$tb = ops("ralph_thresholdB");
			$tc = ops("ralph_thresholdC");
			
			if ($total > $tc && $tc > 1) {
				// Charlie
		    	ralphLog("RALPH", "IP Analysis -> " . acid_to_ip($ip), $flag, "Threshold Charlie Exceeded: Auto Heavy Scan Initiated", acid_to_ip($ip));
		    	addScan($ip, 3);
			} else if ($total > $tb && $tb > 1) {
				// Beta
		    	ralphLog("RALPH", "IP Analysis -> " . acid_to_ip($ip), $flag, "Threshold Beta Exceeded: Auto Heavy Scan Initiated", acid_to_ip($ip));
		    	addScan($ip, 2);
			} else if ($total > $ta && $ta > 1) {
				// Alpha
		    	ralphLog("RALPH", "IP Analysis -> " . acid_to_ip($ip), $flag, "Threshold Alpha Exceeded: Auto Heavy Scan Initiated", acid_to_ip($ip));
		    	addScan($ip, 1);
			}
			
		    ralphLog("RALPH", "IP Analysis -> End", $flag, "No Comment", acid_to_ip($ip));
		}
	}
}

function scanProcessor() {
    global $login, $idsDB;

    procWrite("B", ".. Scanner Started");

    $idsQueryA = $idsDB->query("SELECT id, ip, status, uid, tcp_ports, udp_ports, enabled, timestamp, type FROM idsScanner where status = 3 and enabled = 1 order by timestamp limit 10");
    for ($i = 0; $row = mysql_fetch_row($idsQueryA); $i++) {
        $id[] = $row[0];
        $ip[] = $row[1];
        $status[] = $row[2];
        $uid[] = $row[3];
        $tcp_ports[] = $row[4];
        $udp_ports[] = $row[5];
        $enabled[] = $row[6];
        $timestamp[] = $row[7];
        $type[] = $row[8];
    }

    if (!$id) procWrite("B", " . Nothing to do");

    for ($i=0;$i<count($id);$i++) {
        if ($type[$i] == 1) {
            $success = pingHost($id[$i]);
            procWrite("B", " - Pinging " . acid_to_ip($ip[$i]) . " $success");
        } else if ($type[$i] == 2 || $type[$i] == 3) {
            $success = pingHost($id[$i]);
            procWrite("B", " - Pinging " . acid_to_ip($ip[$i]) . " $success");
            procWrite("B", " - Port Scanning " . acid_to_ip($ip[$i]));
            scanHost($id[$i], $type[$i]);
        }
    }

    procWrite("B", ".. Scanner Finished");
	    
}

function communityBufferProcess() {
    global $ops, $idsDB;

    $myCommunityObj = new myCommunity();

    if (ops("site_community") == 1) {

		procWrite("B", ".. Community Processing Started");

	    $idsQueryA = $idsDB->query("select id, sender, message, timestamp from " . ops("site_table.idsCommunityBuffer") . " order by timestamp desc");
        while (list($d_id, $d_sender, $d_message, $d_timestamp) = mysql_fetch_row($idsQueryA)) {
            transmitCommunityBuffer($d_sender, $d_message);
            $myCommunityObj->communityBufferDelete($d_id);
        }

        $idsQueryA = $idsDB->query("select id from " . ops("site_table.idsCommunityMain") . " order by id desc limit 1");
        while (list($d_id) = mysql_fetch_row($idsQueryA)) {
            $id = $d_id;
	    }
	    
	    if (!$id) {
            receiveCommunityMain(0);
        } else if ($id) {
		    receiveCommunityMain($id);
	    }

	    procWrite("B", " . Community Processing Finished");
    }
}

function transmitCommunityBuffer($sender, $message) {
    global $ops;

    procWrite("B", " . Transmitting Buffer");

	// build payload
	$payload = "$sender||$message";

	// base64 payload data
	$payload = base64_encode($payload);
	$key1 = ops("site_registration");
    $out = exConnect("community.aanval.com", "community_payload.php?payload=$payload&certify=$key1");

	procWrite("B", " . Transmitted: $out");
}

function receiveCommunityMain($id) {
	global $ops, $idsDB;
	
	$epoch = date("U");

	procWrite("B", " . Downloading Messages");
	procWrite("B", " . Starting Message ID: $id");
	
	$key1 = ops("site_registration");
	$out = exConnect("community.aanval.com", "community_receive.php?id=$id&certify=$key1");
	
	// base64 payload data
	$payload = base64_decode($out);
	$payload = explode("|||", $payload);
	
	for ($i=0;$i<count($payload);$i++) {
		$data = explode("||", $payload[$i]);
		$data[1] = strip_tags(addslashes($data[1]));
		$data[2] = strip_tags(addslashes($data[2]));
		
		if (strstr($data[2], "red:")) {
			$data[2] = str_replace("red:", "", $data[2]);
			$data[2] = trim($data[2]);
			$data[2] = "<font color=darkred>$data[2]</font>";
		} else if (strstr($data[2], "orange:")) {
			$data[2] = str_replace("orange:", "", $data[2]);
			$data[2] = trim($data[2]);
			$data[2] = "<font color=darkorange>$data[2]</font>";
		} else if (strstr($data[2], "blue:")) {
			$data[2] = str_replace("blue:", "", $data[2]);
			$data[2] = trim($data[2]);
			$data[2] = "<font color=darkblue>$data[2]</font>";
		} else if (strstr($data[2], "green:")) {
			$data[2] = str_replace("green:", "", $data[2]);
			$data[2] = trim($data[2]);
			$data[2] = "<font color=darkgreen>$data[2]</font>";
		} else if (strstr($data[2], "purple:")) {
			$data[2] = str_replace("purple:", "", $data[2]);
			$data[2] = trim($data[2]);
			$data[2] = "<font color=purple>$data[2]</font>";
		}
		
		if ($data[0]) {
			$idsQueryA = $idsDB->query("insert into " . ops("site_table.idsCommunityMain") . " (id, sender, message, timestamp) values ('$data[0]', '$data[1]', '$data[2]', '$epoch')");
		}
	}
	
}

function sigProcessor($file, $type) {
    global $ops, $idsDB, $root_path;

    if ((ops("site_sigProcessorEnabled") != 1)) return;
    
    procWrite("B", ".. Processing Signatures");
    $epoch = date("U");
    $lastRun = statusSigProcessorLastRunDiff();

    $objStatus = new myStatus();
    $objStatus->loadStatus();
    $last = $objStatus->getStatusVar("sigProcessor_lastRun");

    if (ops("site_sigFileCustom") != "") {
	    $file = ops("site_sigFileCustom");
	    if(file_exists("$root_path/tmp/sig/$file")) $fileFound = 1;
    }    
    
    if ($lastRun > ops("site_sigProcessorFrequency") || ops("site_sigProcessorNow") == 1 || $fileFound == 1) {
        procWrite("B", ".. Signature System Executed");

        // Get Signature File - Save & Untar File
        if (ops("site_sigProcessorSite") == "A") {
            $primarySite = ops("site_sigProcessorSiteA");
        } else {
            $primarySite = ops("site_sigProcessorSiteB");
        }

        // Check for and / or retrieve file
        if (!$fileFound) {
        	if (sigDownload($primarySite, $file) == 0) return;
        }
        @chdir("$root_path/tmp/sig/");
        @exec("gunzip -c $file | tar -xvf -");
        @unlink("$root_path/tmp/sig/$file");

        // Process INTO signature system
        sigDataProcess();

        // Trigger Sensors to retreive new signatures
        $idsQueryA = $idsDB->query("select distinct sensorid from idsSignaturesIndex");
        while (list($d_sid) = mysql_fetch_row($idsQueryA)) {
            $sid = $d_sid;
            $idsQueryB = $idsDB->query("insert into idsSensorManageMessages (sid, message, enabled, issued) values ('$sid', 'SNORTRULES', '1', '$epoch')");
            dblog("SIG_PROCESSOR", "Snort Message Post ($sid)", 0, "SNORTRULESSEND");
            $idsQueryB = $idsDB->query("insert into idsSensorManageMessages (sid, message, enabled, issued) values ('$sid', 'SNORTRESTART', '1', '$epoch')");
            dblog("SIG_PROCESSOR", "Snort Message Post ($sid)", 0, "SNORTRESTART");
        }

        // Save Status & Finish
        if ($type == 2) {
	    	$objStatus = new myStatus();
	    	$objStatus->loadStatus();
	    	$objStatus->setStatusVar("sigProcessor_lastRun", date("U"));

	        // Update SigProcessorNow OPS Setting to 0
	        $ops->setOpsVar("site_sigProcessorNow", 0);
        }

        if (ops("ralph_signatureUpdateEmail") != "") {
            sendSignatureUpdateEmail($last);
        }

        procWrite("B", " . Done!");
    } else {
        procWrite("B", " . Nothing to do");
    }

}

function sigDownload($host, $file) {
    global $root_path;

    $status = 0;

    procWrite("B", " - Downloading from:\n - $host\n - Receiving file:\n - $file");

    $data = exConnect($host, $file);
    $handle = fopen("$root_path/tmp/sig/$file","w");
    fputs($handle, $data);
    fclose($handle);

    if (file_exists("$root_path/tmp/sig/$file")) $status = 1;

    return $status;
}

function sigDataProcess() {
    global $root_path;

    $epoch = date("U");
    $ruleDir = "$root_path/tmp/sig/rules/";

    procWrite("B", " - Processing Signatures");

    if (!$this_dir = @dir("$ruleDir")) {
            procWrite("B", ".. ERROR:<p>$ruleDir not correct or not found!");
    } else {
	    while ($file = $this_dir->read()) {
	        if ($file != "." && $file != "..") {
	            if (strstr($file, ".rules")) {
	                $ruleFile = file("$ruleDir/$file");
	
	                for($i=0;$i<count($ruleFile);$i++) {
	                    sigInsert($ruleFile[$i], $file);
	                }
	            }
	        }
	    }
    }
}

function sendSignatureUpdateEmail($last) {
    global $login, $ops, $idsDB;

    $userid = $login->getSessionVar("id");

    $message = "Signature Updates Report:\n\n";

    $idsQueryA = $idsDB->query("select id, uid, flag, module, action, ip, misc, intime from idsLogs where module = 'SIG' and intime > '$last' order by intime desc");
    for ($i = 0; $row = mysql_fetch_row($idsQueryA); $i++) {
        $message .= "$row[4]:\n$row[6]\n\n";
        $dat = 1;
    }

    procWrite("B", " - Email Report Transmitted");
    if (!$dat) $message = "Signature Updates Report:\n\nNothing updated.\n";

    $to = trim(ops("ralph_signatureUpdateEmail"));
    $subject = ops("ralph_signatureUpdateEmailSubject");
    sendmail(ops("ralph_signatureUpdateEmail"), "Aanval - Signature Updater", $to, $to, $subject, $message);
}

function aiProcessor() {
    global $ops, $idsDB;

    $objSensor = new mySensor();
    $sensorArray = $objSensor->getSensorIDs(1);
    
    if (count($sensorArray) == 0) return;
    
    procWrite("D", ".. AI Processing Started");
	
    foreach ($sensorArray as $sensor) {

	    $epoch = date("U");
    	$objSensor->loadSensor($sensor);
    	
	    procWrite("D", " . Processing Snort Sensor: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]");
    	
	    ### Sensor Status ###
	    
    	if ($objSensor->getSensorVar("countStatusFailure") >= 5 && $objSensor->getSensorVar("countStatusFailure") <= 10) {

    		// Update status count to bypass this section
    		$objSensor->setSensorVar("lastStatusFailure", $epoch);
    		
    		// Email
    		if ($objSensor->getSensorVar("emailOnStatusFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnStatusFailure"));
				$subject = "Snort Restart Attempted: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Problem:\n";
				$message .= "Snort appears to have stopped on sensor '" . $objSensor->getSensorVar("name") . "'.\n\n";
				$message .= "Solution:\n";
                $message .= "The console attempted to restart snort and if successful, you will not receive any further emails. ";
                $message .= "However, if this snort restart fails you will receive additional updates via email and ";
                $message .= "should investigate the problem as soon as possible.\n\n";
                $message .= "---\nStatus Failure Count: " . $objSensor->getSensorVar("countStatusFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countStatusFailure", 11);

    		// Issue snort restart
    		sensorMessagePost("SNORTRESTART", $sensor);
    		
    	} else if ($objSensor->getSensorVar("countStatusFailure") >= 30 && $objSensor->getSensorVar("countStatusFailure") <= 40) {

    		// Email
    		if ($objSensor->getSensorVar("emailOnStatusFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnStatusFailure"));
				$subject = "Snort Status Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 1 of 4:\n---\n\nProblem:\n";
				$message .= "A problem with snort sensor '" . $objSensor->getSensorVar("name") . "' has been detected. ";
                $message .= "This usually indicates the snort process is no longer running on the mentioned sensor.\n\n";
                $message .= "The console has already attempted an automatic restart of snort, however it appears to have failed.\n\n";
				$message .= "Solution:\n";
                $message .= "The console will attempt to restart snort again momentarily, if this fails you will ";
                $message .= "receive an additional warning by email and should begin resolving the problem as quickly as possible.\n\n";
                $message .= "---\nStatus Failure Count: " . $objSensor->getSensorVar("countStatusFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}
    		
    		// Issue snort restart
    		sensorMessagePost("SNORTRESTART", $sensor);
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countStatusFailure", 41);    		
    		
    	} else if ($objSensor->getSensorVar("countStatusFailure") >= 100 && $objSensor->getSensorVar("countStatusFailure") <= 110) {

    		// Email
    		if ($objSensor->getSensorVar("emailOnStatusFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnStatusFailure"));
				$subject = "Snort Status Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 2 of 4:\n---\n\nProblem:\n";
				$message .= "The problem with snort sensor '" . $objSensor->getSensorVar("name") . "' has not been resolved. ";
                $message .= "The snort process appears to no longer be running or the console can no longer detect its presence.\n\n";
                $message .= "The console has already attempted an automatic restart of snort twice, however both attempts appear to have ";
                $message .= "failed and this situation requires human intervention to be resolved.\n\n";
				$message .= "Solution:\n";
                $message .= "The console will continue to monitor snort and wait for the situation to be resolved. If this problem is not ";
                $message .= "resolved within the next 15 - 20 minutes, additional warning emails will be sent as a reminder.\n\n";
                $message .= "---\nStatus Failure Count: " . $objSensor->getSensorVar("countStatusFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countStatusFailure", 111); 
    		    		
    	} else if ($objSensor->getSensorVar("countStatusFailure") >= 460 && $objSensor->getSensorVar("countStatusFailure") <= 470) {

    		// Email
    		if ($objSensor->getSensorVar("emailOnStatusFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnStatusFailure"));
				$subject = "Snort Status Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 3 of 4:\n---\n\nProblem:\n";
				$message .= "The problem with snort sensor '" . $objSensor->getSensorVar("name") . "' remains to be unresolved. ";
                $message .= "The snort process appears to no longer be running or the console can no longer detect its presence.\n\n";
                $message .= "This is a reminder by email that this issue requires human intervention.\n\n";
				$message .= "Solution:\n";
                $message .= "The console will attempt to restart snort one last time, to assist with this problem. However, the console ";
                $message .= "will continue to monitor snort and wait for the situation to be resolved. If this problem is not ";
                $message .= "resolved within the next 15 - 20 minutes, one last email warning will be sent as a reminder.\n\n";
                $message .= "---\nStatus Failure Count: " . $objSensor->getSensorVar("countStatusFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}
    		
    		// Issue snort restart
    		sensorMessagePost("SNORTRESTART", $sensor);
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countStatusFailure", 471);     		
    		
    	} else if ($objSensor->getSensorVar("countStatusFailure") >= 820 && $objSensor->getSensorVar("countStatusFailure") <= 830) {

    		// Email
    		if ($objSensor->getSensorVar("emailOnStatusFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnStatusFailure"));
				$subject = "Snort Status Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 4 of 4:\n---\n\nProblem:\n";
				$message .= "This is your last reminder / warning email.\n\n";
				$message .= "The problem with snort sensor '" . $objSensor->getSensorVar("name") . "' remains to be unresolved. ";
                $message .= "The snort process appears to no longer be running or the console can no longer detect its presence. The process ";
				$message .= "has not been running for quite some time now and needs to be resolved immediately. ";
                $message .= "This is a reminder by email that this issue requires human intervention.\n\n";
				$message .= "Solution:\n";
                $message .= "The console has attempted to restart snort several times and appears to have failed. Human intervention is ";
                $message .= "required at this point in time. This is your last email concerning this sensor status issue at this time.\n\n";
                $message .= "---\nStatus Failure Count: " . $objSensor->getSensorVar("countStatusFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countStatusFailure", 831);
    	}
    	
    	### Comm ###

		if (($epoch - 30) > $objSensor->getSensorVar("hb")) {
    		$objSensor->setSensorVar("lastCommFailure", $epoch);
    		$objSensor->setSensorVar("countCommFailure", $objSensor->getSensorVar("countCommFailure") + 1);
		} else {
    		$objSensor->setSensorVar("countCommFailure", 0);
		}
    	
		if ($objSensor->getSensorVar("countCommFailure") >= 10 && $objSensor->getSensorVar("countCommFailure") <= 20) {
    		$objSensor->setSensorVar("lastCommFailure", $epoch);

    		// Email
    		if ($objSensor->getSensorVar("emailOnCommFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnCommFailure"));
				$subject = "SMT Communication Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 1 of 4:\n---\n\nProblem:\n";
				$message .= "A communication issue with sensor '" . $objSensor->getSensorVar("name") . "' and the console has been detected. ";
                $message .= "This usually indicates a network connectivity related issue has occurred or the sensor management tools ";
                $message .= "have stopped.\n\n";
				$message .= "Solution:\n";
                $message .= "The console will continue to monitor the progress of this issue, however this should be reviewed as soon as ";
                $message .= "possible in the event there is a serious network or system problem.\n\n";
                $message .= "Additional email warnings will be sent shortly if this problem is not resolved.\n\n";
                $message .= "---\nCommunication Failure Count: " . $objSensor->getSensorVar("countCommFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}    		
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countCommFailure", 21);
    		
		} else if ($objSensor->getSensorVar("countCommFailure") >= 100 && $objSensor->getSensorVar("countCommFailure") <= 110) {
    		$objSensor->setSensorVar("lastCommFailure", $epoch);

    		// Email
    		if ($objSensor->getSensorVar("emailOnCommFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnCommFailure"));
				$subject = "SMT Communication Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 2 of 4:\n---\n\nProblem:\n";
				$message .= "A communication issue with sensor '" . $objSensor->getSensorVar("name") . "' remains to be resolved. ";
                $message .= "This usually indicates it is less a network connectivity related issue and a higher chance of it being ";
                $message .= "a server or sensor management tool related issue.\n\n";
				$message .= "Solution:\n";
                $message .= "The console will continue to monitor the progress of this issue, however this should be reviewed as soon as ";
                $message .= "possible in the event there is a serious or critical network or system related issue.\n\n";
                $message .= "Additional email warnings will be sent shortly if this problem is not resolved.\n\n";
                $message .= "---\nCommunication Failure Count: " . $objSensor->getSensorVar("countCommFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}    		
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countCommFailure", 111);

		} else if ($objSensor->getSensorVar("countCommFailure") >= 400 && $objSensor->getSensorVar("countCommFailure") <= 410) {
    		$objSensor->setSensorVar("lastCommFailure", $epoch);
    		
    		// Email
    		if ($objSensor->getSensorVar("emailOnCommFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnCommFailure"));
				$subject = "SMT Communication Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 3 of 4:\n---\n\nProblem:\n";
				$message .= "A communication issue with sensor '" . $objSensor->getSensorVar("name") . "' has yet to be remedied. ";
                $message .= "This prolonged communication outage usually indicates the server is unavailable or the sensor management tools ";
                $message .= "have stopped or failed.\n\n";
				$message .= "Solution:\n";
                $message .= "The console will continue to monitor the progress of this issue, however this should be reviewed as soon as ";
                $message .= "possible in the event there is a critical system related issue going on unnoticed.\n\n";
                $message .= "One more email warning will be sent as a reminder to get this problem resolved.\n\n";
                $message .= "---\nCommunication Failure Count: " . $objSensor->getSensorVar("countCommFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}      		
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countCommFailure", 411);

		} else if ($objSensor->getSensorVar("countCommFailure") >= 800 && $objSensor->getSensorVar("countCommFailure") <= 810) {
    		$objSensor->setSensorVar("lastCommFailure", $epoch);
    		
    		// Email
    		if ($objSensor->getSensorVar("emailOnCommFailure") != "") {
				$to = trim($objSensor->getSensorVar("emailOnCommFailure"));
				$subject = "SMT Communication Failure: " . $objSensor->getSensorVar("id") . " [" . $objSensor->getSensorVar("name") . "]";
				$message = "Priority 4 of 4:\n---\n\nProblem:\n";
				$message .= "This is your last reminder / warning email.\n\n";
				$message .= "The communication issue with sensor '" . $objSensor->getSensorVar("name") . "' continues to be a problem. ";
                $message .= "This prolonged communication outage will affect sensor management functionality if not resolved soon.\n\n";
				$message .= "Solution:\n";
                $message .= "Human intervention is required to identify the specific cause of this communication outage and to resolve it ";
                $message .= "promptly. This is your last email concerning this sensor communication related issue at this time.\n\n";
                $message .= "---\nCommunication Failure Count: " . $objSensor->getSensorVar("countCommFailure") . "\n---\n";
				sendmail("intrusion@console.com", "Intrusion Detection Console", $to, $to, $subject, $message);
    		}      		
    		
    		// Update status count to bypass this section
    		$objSensor->setSensorVar("countCommFailure", 811);
		}    		
    }
	
    procWrite("D", ".. AI Processing Finished");
}

function indexer() {

	// Begin processing syslog
	procWrite("C", ".. Index Processing Started");	    

	// Set time variable
    $tmpStart = date("U");	
	
	// Instantiate new indexer object
    $indexer = new indexer();    

    // Processes Indexes
    $indexer->indexRun();

	// Calculate time differences
    $tmpEnd = date("U");
    $tmpTime = $tmpEnd - $tmpStart;
    
    // End
    procWrite("C", " . Processed in $tmpTime seconds");
    procWrite("C", " . Index Processing Finished");
}

function processReportSchedule() {
    global $ops, $idsDB;

    procWrite("B", ".. Processing Report Schedule");

    $currentEpoch = date("U");
    $todayDate = date("y-m-d");
    $todayDateName = substr(date("l"), 0, 3);
    $todayEpoch = strtotime($todayDate);
    $tomorrowEpoch = strtotime($todayDate) + 86400;

    $dayQuery = strtolower("$todayDateName = 1");
    
    $idsQueryA = $idsDB->query("select id, userid, name, email, query, runtime, runlast from idsReportSchedule where $dayQuery and enabled = 1 and runlast not between $todayEpoch and $tomorrowEpoch order by id desc");
    while (list($id, $userid, $name, $email, $query, $runtime, $runlast) = mysql_fetch_row($idsQueryA)) {

    	$tmpTime = $todayDate . " $runtime";
		$tmpEpoch = strtotime($tmpTime);
		
		// Send report if report send time is less than current time
		if ($tmpEpoch < $currentEpoch) {

            procWrite("B", " . Sending Report [$name]");
            procWrite("B", " . Sending To [$email]");

            // Send Reports
            $fromEmail = $email;
            $toEmail = $email;
            $fromName = "Aanval Console";
            $toName = "Aanval Report Recipient";

			$objDisplay = new mySearchDisplay();
			$objDisplay->setSearchDisplayVar("email", true);
			$objDisplay->setSearchDisplayVar("userid", $userid);
			$objDisplay->setSearchDisplayVar("searchQuery", $query);
			$report = $objDisplay->searchDisplay();
			
            sendmail($fromEmail, $fromName, $toEmail, $toName, "Aanval Email Report: $name", $report);
            ralphLog("RALPH", "Sending Custom report", $flag, $toEmail, "127.0.0.1");
						
		    $idsQueryA = $idsDB->query("update idsReportSchedule set runlast = '$currentEpoch' where id = '$id'");
			
		}
    }

    if (!$report) {
        // Nothing to do
        procWrite("B", " . No Reports Sent");
    }

}

function processDailyReports() {
    global $ops;

    procWrite("B", ".. Processing Daily Reports");

    $currentEpoch = date("U");
    $todayDate = date("y-m-d");
    $todayEpoch = strtotime($todayDate);
    $tomorrowEpoch = strtotime($todayDate + 86400);
    $reportSendTime = ops("site_dailyStatsReportTime");
	$reportSendEpoch = strtotime("$todayDate $reportSendTime");
    $reportLastSendTime = ops("site_dailyStatsReportLast");

    if (ops("site_dailyStatsReportEmail") != "") {
        // Generate and send report to the listed emails

        // Check if ran yet today
        if ($reportLastSendTime < $todayEpoch) {

            // Only run at specified time
            if ($currentEpoch > $reportSendEpoch) {

                procWrite("B", " . Sending Reports");

                // Get Data For Reports
                $statsYesterday = statsYesterday();
                $statsToday = statsToday();
                $statsLast12 = statsLastHours(12);
                $statsLast24 = statsLastHours(24);
                $statsLast72 = statsLastHours(72);
                $statsLast120 = statsLastHours(120);

                // Get Status for each processor
                $objStatus = new myStatus();
                $objStatus->loadStatus();

            	$processorALastRun = statusProcessorLastRun("A");
            	$processorALastRunDiff = statusProcessorLastRunDiff("A");
            	$processorARunCount = statusProcessorRunCount("A");

            	$processorBLastRun = statusProcessorLastRun("B");
            	$processorBLastRunDiff = statusProcessorLastRunDiff("B");
            	$processorBRunCount = statusProcessorRunCount("B");

            	$processorCLastRun = statusProcessorLastRun("C");
            	$processorCLastRunDiff = statusProcessorLastRunDiff("C");
            	$processorCRunCount = statusProcessorRunCount("C");

            	$processorDLastRun = statusProcessorLastRun("D");
            	$processorDLastRunDiff = statusProcessorLastRunDiff("D");
            	$processorDRunCount = statusProcessorRunCount("D");

                // Assess Status for each processor
            	if ($processorALastRunDiff > 1000) {
            		$processorAStatus = "Failure (hung or not running)";
            	} else if ($processorALastRunDiff > 480) {
                    $processorAStatus = "Danger (might be a problem)";
                } else if ($processorALastRunDiff > 240) {
                    $processorAStatus = "Warning (taking longer than usual)";
            	} else {
                    $processorAStatus = "Active";
            	}

            	if ($processorBLastRunDiff > 1000) {
            		$processorBStatus = "Failure (hung or not running)";
            	} else if ($processorBLastRunDiff > 480) {
                    $processorBStatus = "Danger (might be a problem)";
                } else if ($processorBLastRunDiff > 240) {
                    $processorBStatus = "Warning (taking longer than usual)";
            	} else {
                    $processorBStatus = "Active";
            	}

            	if ($processorCLastRunDiff > 1000) {
            		$processorCStatus = "Failure (hung or not running)";
            	} else if ($processorCLastRunDiff > 480) {
                    $processorCStatus = "Danger (might be a problem)";
                } else if ($processorCLastRunDiff > 240) {
                    $processorCStatus = "Warning (taking longer than usual)";
            	} else {
                    $processorCStatus = "Active";
            	}

            	if ($processorDLastRunDiff > 1000) {
            		$processorDStatus = "Failure (hung or not running)";
            	} else if ($processorDLastRunDiff > 480) {
                    $processorDStatus = "Danger (might be a problem)";
                } else if ($processorDLastRunDiff > 240) {
                    $processorDStatus = "Warning (taking longer than usual)";
            	} else {
                    $processorDStatus = "Active";
            	}

                // Send Reports
                $fromEmail = ops("site_dailyStatsReportFrom");
                $toEmail = ops("site_dailyStatsReportEmail");
                $fromName = ops("site_dailyStatsReportFrom");
                $toName = "Aanval Report Recipient";

                $report = "Daily Statistics Report\n---\n\n";
                $report .= "[Total Events by Time Period]\n";
                $report .= "Today: $statsToday\n";
                $report .= "Yesterday: $statsYesterday\n\n";

                $report .= "[Total Events by Last x Hours]\n";
                $report .= "Last 12 Hours: $statsLast12\n";
                $report .= "Last 24 Hours: $statsLast24\n";
                $report .= "Last 72 Hours: $statsLast72\n";
                $report .= "Last 120 Hours: $statsLast120\n\n";

                $report .= "[Console Status]\n";
                $report .= "Background Processor A Status: $processorAStatus\n";
        		$report .= "Background Processor A Last Run: " . timeDisplay($processorALastRun, 1) . " ($processorALastRun)\n";
                $report .= "Background Processor A Last Run Diff: $processorALastRunDiff\n";
                $report .= "Background Processor A Run Count: $processorARunCount\n";
                $report .= "\n";
                $report .= "Background Processor B Status: $processorBStatus\n";
        		$report .= "Background Processor B Last Run: " . timeDisplay($processorBLastRun, 1) . " ($processorBLastRun)\n";
                $report .= "Background Processor B Last Run Diff: $processorBLastRunDiff\n";
                $report .= "Background Processor B Run Count: $processorBRunCount\n";
                $report .= "\n";
                $report .= "Background Processor C Status: $processorCStatus\n";
        		$report .= "Background Processor C Last Run: " . timeDisplay($processorCLastRun, 1) . " ($processorCLastRun)\n";
                $report .= "Background Processor C Last Run Diff: $processorCLastRunDiff\n";
                $report .= "Background Processor C Run Count: $processorCRunCount\n";
                $report .= "\n";
                $report .= "Background Processor D Status: $processorDStatus\n";
        		$report .= "Background Processor D Last Run: " . timeDisplay($processorDLastRun, 1) . " ($processorDLastRun)\n";
                $report .= "Background Processor D Last Run Diff: $processorDLastRunDiff\n";
                $report .= "Background Processor D Run Count: $processorDRunCount\n";
                $report .= "\n";

                sendmail($fromEmail, $fromName, $toEmail, $toName, ops("site_dailyStatsReportSubject"), $report);
                ralphLog("RALPH", "Sending Daily Statistics Report", $flag, $toEmail, "127.0.0.1");

                // Update last sent
                $ops->setOpsVar("site_dailyStatsReportLast", $currentEpoch);
            } else {
                 procWrite("B", " . Not quite time");
            }
        } else {
             procWrite("B", " . Already sent today");
        }

    } else {
        // Nothing to do
        procWrite("B", " . No recipients for reports");
    }

}

function deleteIgnoredEvents() {
	global $login, $ops, $idsDB;

	// Check to see if the delete ignored events option is enabled and exit if not
	if (ops("site_deleteIgnoredEvents") == 0) return;

    procWrite("B", ".. Ignored Event Trimming Started");
	
    // Retrieve array of ignored event id's
    $idsQueryA = $idsDB->query("select id from idsMain where xcol1 = 1");
    while (list($id) = @mysql_fetch_row($idsQueryA)) {
        $data[] = $id;
	}
    
	$count = count($data);
	
    procWrite("B", " . Trimming $count events");
	
    // Loop through each element and delete from the appropriate indexes and idsMain
    for ($i=0;$i<count($data);$i++) {
        $idsQueryA = $idsDB->query("delete from idsMain where id = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndex where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexSip where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexDip where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexAck where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexWin where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexLen where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexProto where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexSensor where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexSeq where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexSport where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexDport where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexTtl where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexTime where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexCategory where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexSignature where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexCategoryName where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexSignatureName where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexLevel where loc = $data[$i]");
        $idsQueryA = $idsDB->query("delete from idsReverseIndexModule where loc = $data[$i]");
    }
    
    procWrite("B", " . Ignored Event Trimming Finished");
    
}

function autoSnortTrimmer() {
	global $ops;
	
	$objModule = new myModule();
	$objModule->loadModule("1");

	// Return if snort module is not enabled
    if ($objModule->getModuleVar("enabled") == 0) {
		return;
    }
    
    // Return if snort auto trimming is disabled
	if (ops("site_snortAutoTrim") == 0) {
		return;
	}

    procWrite("B", ".. Snort Auto Trimming Started");
	
	$moduleDB = new moduleDB;
	$moduleDB->loadDB(1);
		
	$Q = $moduleDB->query("select count(*) as cnt from event");
	if(mysql_num_rows($Q) > 0) {
		$row = mysql_fetch_array($Q);
		$eventTotal = $row["cnt"];
	}
	
	if (ops("site_snortAutoTrimMax") > $eventTotal) {
		// Nothing to be done
	    procWrite("B", " . 0 events to be trimmed");	
	} else {
	
		$diff = $eventTotal - ops("site_snortAutoTrimMax");
		
	    procWrite("B", " . $diff events to be trimmed");	
		
		if ($diff > 5000) {
			$diff = 5000;
		    procWrite("B", " . 5000 events will be trimmed");	
		}
		
		if ($diff > 0) {	
	
			$Q = $moduleDB->query("select sid, cid from event order by timestamp limit $diff");
			if(mysql_num_rows($Q) > 0) {
				while ($row = mysql_fetch_array($Q)) {
					$cnt++;
					$Q2 = $moduleDB->query("delete from event where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
					$Q2 = $moduleDB->query("delete from data where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
					$Q2 = $moduleDB->query("delete from tcphdr where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
					$Q2 = $moduleDB->query("delete from udphdr where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
					$Q2 = $moduleDB->query("delete from iphdr where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
					$Q2 = $moduleDB->query("delete from icmphdr where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
					$Q2 = $moduleDB->query("delete from opt where sid = '" . $row["sid"] . "' and cid = '" . $row["cid"] . "'");
				}
				
			    procWrite("B", " . $cnt events trimmed this run");	
			}		
	
		} else {
			
		}
	}
	
	$moduleDB->close();	
	
    procWrite("B", " . Snort Auto Trimming Finished");
}

function checkModule($module) {

    $array = get_loaded_extensions();
    
    for ($i=0;$i<count($array);$i++) {
        if (strstr($array[$i], $module)) {
            $f = 1;
        }
    }

    if ($f == 1) {
		return 1;
    } else {
		return 0;
    }

    // should never reach here
    return 0;

}

function procWrite($processor, $data) {
	global $ops, $root_path;
	
	$data = strip_tags(addslashes($data));
	
	if ($data == "RESET") {
		// Open File and Clear
        if ($fp = @fopen("$root_path" . "tmp/processor$processor", "w")) {

        	// Write procWrite data
        	fputs ($fp, "\n");
        	
        	// Close file handle
            fclose($fp);
        }	
	} else {
		
		// Open File and Append
        if ($fp = @fopen("$root_path" . "tmp/processor$processor", "a")) {

        	// Write procWrite data
        	fputs ($fp, "$data\n");

        	// Close file handle
            fclose($fp);
        }
		
		echo "$data\n";
	}
}

?>

